# Documentation
## Basic information
__Sokoban__ is a logic game in which player tries to move barrels on marked places in a maze.

This application for __Windows__ contains basic engine for standard Sokoban game (single player). To play the game is necessary to be logged in as a user (text box + button __login__). The name of user
may not contain the character `;`.
The next step is to load a game pack (button __load__) which consist of several map. The game progress is saved for the logged user within the map pack.
Character (Sokoban) is controlled by the arrows. There is a timer which measures the length of solving map. The best time for map and the best time for logged user is recorded.
It is possible to let the program solve loaded map (button __solve__) and then to play the solution step by step (button __next__). To restart current map there is a button __restart__.
There is also an option to load map pack which contains the maps with two Sokoban (multiplayer game) but there is no solver implemented yet for such game.
The second sokoban is controlled by the keys __w__, __a__, __s__, __d__.

## Data files for maps
### %Map pack file
Maps are grouped to map pack. The map pack file consists of the identifier on the first line, type of maps on the second line (__SP__ for single player, __MP__ for two player) and
the paths to a map data files. Every path has to be on the separate line. The identifier must not contain the characters `;` nor `+`.

There is two map pack files as examples. The first _map_pack_1.txt_ contains single player maps, the second  _map_pack_2.txt_ contains maps for two player.

### %Map file
%Map is stored in text format. Each map is in a separate file. Each line represents one row of the map grid. The elements of map are represented by digits where
_ground_ = 0, _target for barrel_ = 1, _barrel_ = 2, _barrel on target_ = 3, _sokoban_ = 4, _sokoban on target_ = 5, _wall_ = 6, _grass_ = 7, _second sokoban_ = 8, _second sokoban on target_ = 9.
The map has to be rectangular. To fulfill this requirement there is the element _grass_. At the end of file there has to be `x` on a separate line. Here is an example of the %map file.

6667777<br>
6166666<br>
6110006<br>
6022246<br>
6000066<br>
6666667<br>
x

## Used library
The GUI is provided by the library [Cinder](https://libcinder.org/).

## Compilation
There is _Visul Studio .sln_ project in folder _vc2013_.

## Implementation overview
The main class of this app is SokobanApp which represents the window and the user interface.

### Representation of %map
Maps are represented by the abstract class Map which provides an interface for SokobanApp and it is the ancestor for SinglePlayerMap and TwoPlayerMap.
Game grid is represented by the array [game_grid](class_map.html#af98782081881f6e6c3478011fcdff56c) which contains game elements whose position
is not changed during playing (ground, target for barrel, wall and grass).

The current game configuration for %map is stored in the structure GameState which contains the position of sokoban, list of barrels for simple recognition the dead position -
position when the barrel is not possible to move anymore (it is used in solving), barrel grid for simple recognition if there is a barrel on the position x,y (barrel is represented by
its index to the list of barrels + 1) and hash which is the string with the same information as the barrel grid but every barrels have the same representation (not the index). Hash
also contains information of Sokoban position and it is used in solving to determine which game configurations were visited. Map solving is realized by backtracking. 
For the two player map there is a special game state which includes position of the second sokoban of course. Since solver for two player map is not implemeted, hash and barrel list is 
not used yet in such game state.

### Representation of %user
%User is represented by the class User which stores user's progress in the collection [map](http://www.cplusplus.com/reference/map/) indexed by the %map indentifier. 
For writing progress to the data file there is implemeted operator `<<`.

### Representation of %highscore
%Highscore is represented by the class Highscore. It consists of two collections [std::unordered_map](http://www.cplusplus.com/reference/unordered_map/). 
The first maps the name of `%map name` (`"mapPackIdentifier+numberOfMap"`) to the name of `%user`
which has the best highscore for this %map. The second one maps user and `%map name` to the time of `user` on this `map`. That means it is possible to extract the best time for `map` and `user`
which has this best time and also the best time of `user` on `map`.

### Timer
To measure time there is a class MyTimer using [std::chrono](http://www.cplusplus.com/reference/chrono/). It has the method for getting current time since start in string (format __hh:mm:ss__).
Since there is more demands for current time during
a second, this class remebers the last value. If it is the same as current time there is no redundant conversion to the string.