#pragma once
#include <sstream> //stringstream
#include <iomanip> //setfill, setw
#include <vector>

/**
*Namespace which contains utility functions which are used from more classes in the project.
*/
namespace MyTools {
	/**
	*Splits string according to the separator.
	*
	*@param s Reference to the input string which is splitted.
	*@param separator Separator.
	*@param elems Reference to the list of string which represents parsed input.
	*/
	inline void split(const std::string & s, char separator, std::vector<std::string> & elems) {
		//split string to vector of string by delimiter
		std::stringstream ss;
		ss.str(s);
		std::string item;
		while (std::getline(ss, item, separator)) {
			elems.push_back(item);
		}
	}

	/**
	*Converts string to integer. If string is not convertible to int exception is thrown.
	*
	*@param str String which should be converted to int.
	*@throws std::invalid_argument Number is out of range or string is not convertible to int.
	*@return integer value
	*/
	inline int str_to_int(const std::string & str) {
		int ret;
		std::string::size_type sz;   // alias of size_t

		try {
			ret = std::stoi(str, &sz);
		}
		catch (std::out_of_range) {
			throw std::invalid_argument("too many numbers for integer");
		}
		if (sz != str.length()) //number of parsed characters is less than length of string
			throw std::invalid_argument("cannot convert string to int");
		return ret;
	}

	/**
	*Converts seconds to hh:mm:ss format and writes it to a strigstream.
	*
	*@param sec Seconds.
	*@param ss Reference to the stringstream which is function writes to.
	*/
	inline void sec_to_string_stream(uint64_t sec, std::stringstream & ss) {
		//count information
		uint64_t hours = sec / 3600;
		sec %= 3600;
		uint64_t minutes = sec / 60;
		sec %= 60;
		//write it to a stringsream
		ss << std::setfill('0') << std::setw(2) << hours << ':'
			<< std::setfill('0') << std::setw(2) << minutes << ':'
			<< std::setfill('0') << std::setw(2) << sec;
	}
}