#pragma once
#include<vector>
#include<string>

/**
*Struct which represents a barrel.
*/
struct Barrel {
	/**
	*Constructor. Sets default values for variables
	*/
	Barrel(int x_p, int y_p) : x(x_p), y(y_p) {}

	/**
	*Determines if barrel is equal to another.
	*
	*@param b Another barrel.
	*@return Bool variable determining if barrel is equal to another.
	*/
	bool operator == (const Barrel & b) const {
		return x == b.x && b.y == y;
	}
	int x; /*!< Horizontal position of the barrel.*/
	int y; /*!< Vertical position of the barrel.*/
};

/**
*Structure representing Sokoban game state.
*/
struct GameState {
	/**
	*Constructor. Sets default values for variables.
	*/
	GameState() : sokoban_x(-1), sokoban_y(-1), barrels_on_target(0), hash("") {}
	int sokoban_x; /*!< Horizontal position of Sokoban.*/
	int sokoban_y; /*!< Vertical position of Sokoban.*/
	int barrels_on_target; /*!< Number of barrels which lies on target.*/
	/**
	*The grid which for every coordinate determines if there is a barrel.
	*The values of grid may be 0 (no barrel) or <= 1 (index of barrel in the list of barrels + 1).
	*/
	std::vector<std::vector<int>> barrel_grid; 
	std::vector<Barrel> list_of_barrels; /*!< List of barrels which are on map.*/
	std::string hash;/*!< Hash for this game state - for backtracking.*/
};

/**
*Structure representing Sokoban game state for two player map.
*/
struct TwoPlayerGameState : public GameState {
	/**
	*Constructor. Sets default values for variables
	*/
	TwoPlayerGameState() : sokoban2_x(-1), sokoban2_y(-1) {}
	int sokoban2_x; /*!< Horizontal position of the second Sokoban.*/
	int sokoban2_y;	/*!< Vertical position o the secondf Sokoban.*/
};
