#pragma once
#include "MyTools.h" //sec_to_string
#include <chrono>
#include <string>
#include <sstream>
#include <iomanip>

/**
*Time measurement class.
*/
class MyTimer {
public:
	/**
	*Constructor. Sets defalt values.
	*/
	MyTimer() : old_value(0), old_time_str("00:00:00"), stopped(true) {

	};

	/**
	*Starts time measurement
	*/
	void start_clock() {
		start = std::chrono::system_clock::now();
		stopped = false;
	}

	/**
	*Stops clock and get number of seconds since start.
	*@return Number of seconds since start.
	*/
	uint64_t stop_clock() {
		if (stopped)
			return old_value;
		stopped = true;
		//count information
		uint64_t sec = std::chrono::duration_cast<std::chrono::seconds>(std::chrono::system_clock::now() - start).count();
		//count information
		if (old_value != sec) {
			std::stringstream ss;
			MyTools::sec_to_string_stream(sec, ss);
			old_time_str = ss.str();
			old_value = sec;
		}
		return sec;
	}

	/**
	*Gets time in format hh:mm:ss of seconds since start or between start and stop.

	*@return Time in string in format hh:mm:ss since start or between start and stop.
	*/
	const std::string & get_time() {
		if (stopped)
			return old_time_str;
		uint64_t sec = std::chrono::duration_cast<std::chrono::seconds>(std::chrono::system_clock::now() - start).count();
		if (old_value == sec)
			return old_time_str;
		std::stringstream ss;
		MyTools::sec_to_string_stream(sec, ss); //convert sec to string
		old_time_str = ss.str();
		old_value = sec;
		return old_time_str;

	}


private:
	uint64_t old_value; /*!< Last asked second value*/
	std::string old_time_str; /*!< Last asked string value corresponding to old_value*/
	std::chrono::time_point<std::chrono::system_clock> start; /*!< Clock*/
	//std::chrono::time_point<std::chrono::system_clock> end;
	bool stopped; /*!< bool value representing if timer is stopped*/
};