#pragma once
#include <string> //string
#include <map> //map
#include <sstream> // stringstream

/**
*Represents the user especially his progress in map packs.
*/
class User {
public:
	/**
	*Constructor. Creates an instance from user's name.
	*
	*@param p_name Name of user.
	*/
	User(const std::string & p_name) : name(p_name) {}

	/**
	*Gets progres for map pack of user which is determined by his name.
	*
	*@return Number of finished maps in map pack.
	*/
	int get_progress(const std::string & m_name) {
		return progress[m_name];
	}

	/**
	*Update progress for user and map pack if input parameter representing number of finished level is higher
	*then current progress.
	*
	*@param m_name Name of the map pack.
	*@param lvl Finished level.
	*/
	void set_progress(const std::string & m_name, int lvl) {
		if (lvl > progress[m_name])
			progress[m_name] = lvl;
	}

	/**
	*Gets name of the user which is represented by the instance of this class.
	*
	*@return Name of the user.
	*/
	const std::string & get_name() const{
		return name;
	}

	/**
	Operator <<. Writes information about user's progress.

	@param stream Output stream.
	@param user  User whose progress should be written.
	*/
	friend std::ostream &operator<<(std::ostream & stream, const User & user); //define operator

private:
	std::string name; /*!< name of user*/
	std::map<std::string, int> progress;  /*!< map of "map pack name" and user's progress in this pack */
	/**
	*Operator <. Compare two users according to their names.
	*
	*@param left_u Left operand.
	*@param right_u Right operand.
	*@return Bool variable representing if left operand is less then the right.
	*/
	friend bool operator<(const User & left_u, const User & right_u);
	/**
	*Operator ==. 
	*
	*@param left_u Left operand.
	*@param right_u Right operand.
	*@return Bool variable representing if left operand is qeual to the right.
	*/
	friend bool operator==(const User & left_u, const User & right_u);
};
